@@ -7,53 +7,54 @@
 namespace symspellcpppy {
 
     int SymSpell::MaxDictionaryEditDistance() const {
-        return this->maxDictionaryEditDistance;
+        return maxDictionaryEditDistance;
     }
 
     int SymSpell::PrefixLength() const {
-        return this->prefixLength;
+        return prefixLength;
     }
 
     int SymSpell::MaxLength() const {
-        return this->maxDictionaryWordLength;
+        return maxDictionaryWordLength;
     }
 
     long SymSpell::CountThreshold() const {
-        return this->countThreshold;
+        return countThreshold;
     }
 
     int SymSpell::WordCount() {
-        return this->words.size();
+        return words.size();
     }
 
     int SymSpell::EntryCount() {
-        return this->deletes->size();
+        return deletes->size();
     }
 
-    SymSpell::SymSpell(int initialCapacity, int maxDictionaryEditDistance, int prefixLength, int countThreshold,
-                       unsigned char compactLevel) {
-        if (initialCapacity < 0) throw std::invalid_argument("initialCapacity");
-        if (maxDictionaryEditDistance < 0) throw std::invalid_argument("maxDictionaryEditDistance");
-        if (prefixLength < 1 || prefixLength <= maxDictionaryEditDistance) throw std::invalid_argument("prefixLength");
-        if (countThreshold < 0) throw std::invalid_argument("countThreshold");
-        if (compactLevel > 16) throw std::invalid_argument("compactLevel");
+    SymSpell::SymSpell(int _maxDictionaryEditDistance, int _prefixLength, int _countThreshold, int _initialCapacity,
+                       unsigned char _compactLevel) :
+            maxDictionaryEditDistance(_maxDictionaryEditDistance),
+            prefixLength(_prefixLength),
+            countThreshold(_countThreshold),
+            initialCapacity(_initialCapacity) {
+        if (_initialCapacity < 0) throw std::invalid_argument("initial_capacity is too small.");
+        if (_maxDictionaryEditDistance < 0) throw std::invalid_argument("max_dictionary_edit_distance cannot be negative");
+        if (_prefixLength < 1 || _prefixLength <= _maxDictionaryEditDistance)
+            throw std::invalid_argument("prefix_length cannot be less than 1 or smaller than max_dictionary_edit_distance");
+        if (_countThreshold < 0) throw std::invalid_argument("count_threshold cannot be negative");
+        if (_compactLevel > 16) throw std::invalid_argument("compact_level cannot be greater than 16");
 
-        this->initialCapacity = initialCapacity;
-        this->words.reserve(initialCapacity);
-        this->maxDictionaryEditDistance = maxDictionaryEditDistance;
-        this->prefixLength = prefixLength;
-        this->countThreshold = countThreshold;
-        if (compactLevel > 16) compactLevel = 16;
-        this->compactMask = (UINT_MAX >> (3 + compactLevel)) << 2;
-        this->maxDictionaryWordLength = 0;
-        this->words = std::unordered_map<xstring, int64_t>(initialCapacity);
+        words.reserve(initialCapacity);
+        if (_compactLevel > 16) _compactLevel = 16;
+        compactMask = (UINT_MAX >> (3 + _compactLevel)) << 2;
+        maxDictionaryWordLength = 0;
+        words = std::unordered_map<xstring, int64_t>(initialCapacity);
     }
 
     bool SymSpell::CreateDictionaryEntry(const xstring &key, int64_t count,
                                          const std::shared_ptr<SuggestionStage> &staging) {
 
         if (count <= 0) {
-            if (this->countThreshold > 0)
+            if (countThreshold > 0)
                 return false; // no point doing anything if count is zero, as it can't change anything
             count = 0;
         }
@@ -66,22 +67,22 @@ namespace symspellcpppy {
             if (count >= countThreshold) {
                 belowThresholdWords.erase(key);
             } else {
-                belowThresholdWords.insert(std::pair<xstring, int64_t>(key, count));
+                belowThresholdWords[key] = count;
                 return false;
             }
         } else if (wordsFinded != words.end()) {
             countPrevious = wordsFinded->second;
             count = (MAXINT - countPrevious > count) ? countPrevious + count : MAXINT;
-            words.insert(std::pair<xstring, int64_t>(key, count));
+            words.at(key) = count;
             return false;
         } else if (count < CountThreshold()) {
-            belowThresholdWords.insert(std::pair<xstring, int64_t>(key, count));
+            belowThresholdWords[key] = count;
             return false;
         }
 
         words.insert(std::pair<xstring, int64_t>(key, count));
 
-        if (key.size() > this->maxDictionaryWordLength) this->maxDictionaryWordLength = key.size();
+        if (key.size() > maxDictionaryWordLength) maxDictionaryWordLength = key.size();
 
         //create deletes
         auto edits = EditsPrefix(key);
@@ -97,15 +98,13 @@ namespace symspellcpppy {
                 std::vector<xstring> suggestions;
                 if (deletesFinded != deletes->end()) {
                     suggestions = deletesFinded->second;
-                    std::vector<xstring> newSuggestions(suggestions.size() + 1);
-                    for (int id = 0; id < suggestions.size(); id++) {
-                        newSuggestions[id] = suggestions[id];
-                    }
-
-                    (*deletes)[deleteHash] = suggestions = newSuggestions;
+                    std::vector<xstring> newSuggestions;
+                    newSuggestions.reserve(suggestions.size() + 1);
+                    std::copy(suggestions.begin(), suggestions.end(), std::back_inserter(newSuggestions));
+                    deletes->at(deleteHash) = suggestions = newSuggestions;
                 } else {
                     suggestions = std::vector<xstring>(1);
-                    (*deletes).insert(std::pair<int, std::vector<xstring>>(deleteHash, suggestions));
+                    deletes->insert(std::pair<int, std::vector<xstring>>(deleteHash, suggestions));
                 }
                 suggestions[suggestions.size() - 1] = key;
             }
@@ -115,6 +114,32 @@ namespace symspellcpppy {
         return true;
     }
 
+    bool SymSpell::DeleteDictionaryEntry(const std::string &key) {
+        auto wordsFinded = words.find(key);
+        if (wordsFinded != words.end()) {
+            words.erase(wordsFinded);
+            if(wordsFinded->first.size() == maxDictionaryWordLength) {
+                int max_size = 0;
+                for (auto& word: words) {
+                    max_size = std::max(static_cast<int>(word.first.size()), max_size);
+                }
+                maxDictionaryWordLength = max_size;
+            }
+            auto edits = EditsPrefix(key);
+            for (const auto &edit: *edits) {
+                int deleteHash = GetstringHash(edit);
+                auto deletesFinded = deletes->find(deleteHash);
+                if (deletesFinded != deletes->end()) {
+                    auto delete_vec = deletesFinded->second;
+                    auto it = std::find (delete_vec.begin(), delete_vec.end(), key);
+                    if (it < delete_vec.end()) delete_vec.erase(it);
+                }
+            }
+            return true;
+        }
+        return false;
+    }
+
     bool
     SymSpell::LoadBigramDictionary(const std::string &corpus, int termIndex, int countIndex, xchar separatorChars) {
         xifstream corpusStream;
@@ -191,18 +216,20 @@ namespace symspellcpppy {
             while (getline(ss, token, separatorChars))
                 lineParts.push_back(token);
             if (lineParts.size() >= 2) {
-                int64_t count = stoll(lineParts[countIndex]);
-
+                int64_t count = 1;
+                try {
+                    count = std::stoll(lineParts[countIndex]);
+                } catch (const std::invalid_argument&) {
+                    // Do nothing
+                }
                 CreateDictionaryEntry(lineParts[termIndex], count, staging);
             } else {
                 CreateDictionaryEntry(line, 1, staging);
             }
 
         }
-        if (this->deletes == nullptr)
-            this->deletes = std::make_shared<std::unordered_map<int, std::vector<xstring>>>(staging->DeleteCount());
         CommitStaged(staging);
-        if (this->EntryCount() == 0)
+        if (EntryCount() == 0)
             return false;
         return true;
     }
@@ -228,10 +255,8 @@ namespace symspellcpppy {
             }
 
         }
-        if (this->deletes == nullptr)
-            this->deletes = std::make_shared<std::unordered_map<int, std::vector<xstring>>>(staging->DeleteCount());
         CommitStaged(staging);
-        if (this->EntryCount() == 0)
+        if (EntryCount() == 0)
             return false;
         return true;
     }
@@ -241,11 +266,13 @@ namespace symspellcpppy {
     }
 
     void SymSpell::CommitStaged(const std::shared_ptr<SuggestionStage> &staging) {
+        if (deletes == nullptr)
+            deletes = std::make_shared<std::unordered_map<int, std::vector<xstring>>>(staging->DeleteCount());
         staging->CommitTo(deletes);
     }
 
     std::vector<SuggestItem> SymSpell::Lookup(xstring input, Verbosity verbosity) {
-        return Lookup(std::move(input), verbosity, this->maxDictionaryEditDistance, false);
+        return Lookup(std::move(input), verbosity, maxDictionaryEditDistance, false);
     }
 
     std::vector<SuggestItem> SymSpell::Lookup(xstring input, Verbosity verbosity, int maxEditDistance) {
@@ -255,11 +282,11 @@ namespace symspellcpppy {
     std::vector<SuggestItem>
     SymSpell::Lookup(xstring input, Verbosity verbosity, int maxEditDistance, bool includeUnknown) {
         int skip = 0;
-        if (maxEditDistance > this->maxDictionaryEditDistance) throw std::invalid_argument("maxEditDistance");
+        if (maxEditDistance > maxDictionaryEditDistance) throw std::invalid_argument("Distance too large");
 
         std::vector<SuggestItem> suggestions;
         int inputLen = input.size();
-        if (inputLen - maxEditDistance > this->maxDictionaryWordLength) skip = 1;
+        if (inputLen - maxEditDistance > maxDictionaryWordLength) skip = 1;
 
         int64_t suggestionCount = 0;
         if (words.count(input) && !skip) {
@@ -287,7 +314,7 @@ namespace symspellcpppy {
             } else {
                 candidates.push_back(input);
             }
-            auto distanceComparer = EditDistance(this->distanceAlgorithm);
+            auto distanceComparer = EditDistance(distanceAlgorithm);
             while (candidatePointer < candidates.size()) {
                 xstring candidate = candidates[candidatePointer++];
                 int candidateLen = candidate.size();
@@ -436,7 +463,7 @@ namespace symspellcpppy {
         return deleteWords;
     }
 
-    std::shared_ptr<std::unordered_set<xstring>> SymSpell::EditsPrefix(xstring key) {
+        std::shared_ptr<std::unordered_set<xstring>> SymSpell::EditsPrefix(xstring key) {
         auto m = std::make_shared<std::unordered_set<xstring>>();
         if (key.size() <= maxDictionaryEditDistance) m->insert(XL(""));
         if (key.size() > prefixLength) key = key.substr(0, prefixLength);
@@ -450,7 +477,7 @@ namespace symspellcpppy {
         int lenMask = len;
         if (lenMask > 3) lenMask = 3;
 
-        uint hash = 2166136261;
+        unsigned int hash = 2166136261;
         for (auto i = 0; i < len; i++) {
             {
                 hash ^= s[i];
@@ -458,13 +485,13 @@ namespace symspellcpppy {
             }
         }
 
-        hash &= this->compactMask;
+        hash &= compactMask;
         hash |= (uint) lenMask;
         return (int) hash;
     }
 
     std::vector<SuggestItem> SymSpell::LookupCompound(const xstring &input) {
-        return LookupCompound(input, this->maxDictionaryEditDistance);
+        return LookupCompound(input, maxDictionaryEditDistance);
     }
 
     std::vector<SuggestItem> SymSpell::LookupCompound(const xstring &input, int editDistanceMax) {
@@ -472,7 +499,7 @@ namespace symspellcpppy {
 
         std::vector<SuggestItem> suggestions;     //suggestions for a single term
         std::vector<SuggestItem> suggestionParts; //1 line with separate parts
-        auto distanceComparer = EditDistance(this->distanceAlgorithm);
+        auto distanceComparer = EditDistance(distanceAlgorithm);
 
         bool lastCombi = false;
         for (int i = 0; i < termList1.size(); i++) {
@@ -535,7 +562,7 @@ namespace symspellcpppy {
                                 }
 
                                 suggestionSplit.distance = distance2;
-                                if (bigrams.count(suggestionSplit.term)) {
+                                if (bigrams.count(suggestionSplit.term) > 0) {
                                     long bigramCount = bigrams.at(suggestionSplit.term);
                                     suggestionSplit.count = bigramCount;
                                     if (!suggestions.empty()) {
@@ -561,7 +588,7 @@ namespace symspellcpppy {
                                                                                 (double) suggestions2[0].count));
                                 }
 
-                                if (suggestionSplit.count > suggestionSplitBest.count)
+                                if (suggestionSplitBest.count == 0 || (suggestionSplit.count > suggestionSplitBest.count))
                                     suggestionSplitBest.set(suggestionSplit);
                             }
                         }
@@ -606,14 +633,20 @@ namespace symspellcpppy {
     }
 
     Info SymSpell::WordSegmentation(const xstring &input) {
-        return WordSegmentation(input, this->MaxDictionaryEditDistance(), this->maxDictionaryWordLength);
+        return WordSegmentation(input, MaxDictionaryEditDistance(), maxDictionaryWordLength);
     }
 
     Info SymSpell::WordSegmentation(const xstring &input, int maxEditDistance) {
-        return WordSegmentation(input, maxEditDistance, this->maxDictionaryWordLength);
+        return WordSegmentation(input, maxEditDistance, maxDictionaryWordLength);
     }
 
     Info SymSpell::WordSegmentation(const xstring &input, int maxEditDistance, int maxSegmentationWordLength) {
+        // v6.7
+        // normalize ligatures:
+        // "scientific"
+        // "scientiﬁc" "ﬁelds" "ﬁnal"
+        // TODO: Figure out how to do the below utf-8 normalization in C++.
+        // input = input.Normalize(System.Text.NormalizationForm.FormKC).Replace("\u002D", "");//.Replace("\uC2AD","");
         int arraySize = std::min(maxSegmentationWordLength, (int) input.size());
         std::vector<Info> compositions = std::vector<Info>(arraySize);
         int circularIndex = -1;
@@ -638,9 +671,22 @@ namespace symspellcpppy {
                 part = regex_replace(part, r, XL(""));
                 topEd -= part.size();
 
-                std::vector<SuggestItem> results = this->Lookup(part, Top, maxEditDistance);
+                //v6.7
+                //Lookup against the lowercase term
+                auto partLower = part;
+                std::transform(part.begin(), part.end(), partLower.begin(), ::tolower);
+                std::vector<SuggestItem> results = Lookup(partLower, Top, maxEditDistance);
                 if (!results.empty()) {
                     topResult = results[0].term;
+
+                    //v6.7
+                    //retain/preserve upper case
+                    if (std::isupper(part[0]))
+                    {
+                        topResult[0] = std::toupper(topResult[0]);
+                    }
+
+
                     topEd += results[0].distance;
                     topProbabilityLog = log10((double) results[0].count / (double) N);
                 } else {
@@ -651,22 +697,36 @@ namespace symspellcpppy {
 
                 int destinationIndex = ((i + circularIndex) % arraySize);
 
+                auto circular_distance = compositions[circularIndex].getDistance();
+                auto destination_distance = compositions[destinationIndex].getDistance();
+                auto circular_probablity = compositions[circularIndex].getProbability();
+                auto destination_probablity = compositions[destinationIndex].getProbability();
+
                 if (j == 0) {
                     compositions[destinationIndex].set(part, topResult, topEd, topProbabilityLog);
                 } else if ((i == maxSegmentationWordLength)
-                           || (((compositions[circularIndex].getDistance() + topEd ==
-                                 compositions[destinationIndex].getDistance()) \
- || (compositions[circularIndex].getDistance() + separatorLength + topEd ==
-     compositions[destinationIndex].getDistance())) \
- && (compositions[destinationIndex].getProbability() <
-     compositions[circularIndex].getProbability() + topProbabilityLog))
-                           || (compositions[circularIndex].getDistance() + separatorLength + topEd <
-                               compositions[destinationIndex].getDistance())) {
-                    xstring seg = compositions[circularIndex].getSegmented() + XL(" ") + part;
-                    xstring correct = compositions[circularIndex].getCorrected() + XL(" ") + topResult;
-                    int d = compositions[circularIndex].getDistance() + separatorLength + topEd;
-                    double prob = compositions[circularIndex].getProbability() + topProbabilityLog;
-                    compositions[destinationIndex].set(seg, correct, d, prob);
+                           || (((circular_distance + topEd == destination_distance)
+                                || (circular_distance + separatorLength + topEd == destination_distance))
+                               && (destination_probablity < circular_probablity + topProbabilityLog))
+                           || (circular_distance + separatorLength + topEd < destination_distance)) {
+                    //v6.7
+                    //keep punctuation or spostrophe adjacent to previous word
+                    if (((topResult.size() == 1) && std::ispunct(topResult[0])) || ((topResult.size() == 2) &&
+                                                                                          topResult.rfind("’", 0) == 0)) {
+                        xstring seg = compositions[circularIndex].getSegmented() + part;
+                        xstring correct = compositions[circularIndex].getCorrected() + topResult;
+                        int d = circular_distance        + topEd;
+                        double prob = circular_probablity + topProbabilityLog;
+                        compositions[destinationIndex].set(seg, correct, d, prob);
+                    }
+                    else {
+                        xstring seg = compositions[circularIndex].getSegmented() + XL(" ") + part;
+                        xstring correct = compositions[circularIndex].getCorrected() + XL(" ") + topResult;
+                        int d = circular_distance + separatorLength + topEd;
+                        double prob = circular_probablity + topProbabilityLog;
+                        compositions[destinationIndex].set(seg, correct, d, prob);
+                    }
+
                 }
             }
             circularIndex++;
